import json
import logging

from typing import Optional
from pydantic import BaseModel, Field
from pathlib import Path

from ..base_agent import BaseAgent, AgentConfig
from .software_engineer_prompt import *
from .repo_manager import RepoManager, FileAction

from ...nodes import (
    DocumentLoaderNode,
    OpenAINode,
    ChatInput,
    Message,
    GitRepoNode,
    GitRemoteRepositoryInput,
)

from ....utils.output_parser import LLMOutputParser


REPO_PATH = Path("src/data/git")


software_engineer_config = {
    "name": "software_engineer",
    "description": "A agent for software engineer.",
}


class Plan(BaseModel):
    action: str = Field(description="Plan action.")
    file_path: str = Field(description="File path for action.")
    description: str = Field(description="Description for action.")

    def to_markdown(self):
        action_str = self.action
        match self.action:
            case "add":
                action_str = f"**Add** a new file at {self.file_path}"
            case "remove":
                action_str = f"**Remove** file at {self.file_path}"
            case "modify":
                action_str = f"**Modify** file at {self.file_path}"

        return f"""
- {action_str}
- {self.description}
"""

    def __str__(self):
        action_str = ""
        match self.action:
            case "add":
                action_str = f"Add a new file at {self.file_path}"
            case "remove":
                action_str = f"Remove file at {self.file_path}"
            case "modify":
                action_str = f"Modify file at {self.file_path}"

        return action_str


class SoftwareEngineerAgent(BaseAgent):
    config: AgentConfig = AgentConfig(**software_engineer_config)

    def __init__(self):
        self.document_loader_node = DocumentLoaderNode()
        self.feature_description = ""
        self.plans: list[Plan] = []
        self.repo_url = ""
        self.git_node = GitRepoNode()
        self.repo_manager: Optional[RepoManager] = None
        self.file_actions: list[FileAction] = []
        self._init_openai_node()

    def set_repo_url(self, repo_url: str):
        """
        Set repo url.
        """
        self.repo_url = repo_url
        repo_path = REPO_PATH / Path(repo_url).name
        self.git_node.git_clone(
            input=GitRemoteRepositoryInput(
                url=repo_url,
                path=repo_path,
            )
        )
        self.repo_manager = RepoManager(repo_path)

        # repo structure prompt
        self.openai_node.add_single_message(
            Message(
                role="system",
                content=REPO_STRUCTURE_PROMPT.format(
                    file_tree=self.repo_manager.file_tree_str,
                ),
            )
        )

    def set_local_repo(self, repo_name: str):
        """
        Set local repo path.
        """
        repo_path = REPO_PATH / repo_name
        self.repo_manager = RepoManager(repo_path)

        # repo structure prompt
        self.openai_node.add_single_message(
            Message(
                role="system",
                content=REPO_STRUCTURE_PROMPT.format(
                    file_tree=self.repo_manager.file_tree_str,
                ),
            )
        )

    def set_feature_description(self, feature_description: str):
        """
        Set feature description.
        """
        # add feature description system message to openai node
        self.openai_node.add_single_message(
            Message(
                role="system",
                content=FEATURE_PROMPT.format(feature_description=feature_description),
            )
        )

        self.feature_description = feature_description

    def set_focus_files(self):
        """
        Set focus files of agent for feature development.
        """
        resp = self.openai_node.chat(
            input=ChatInput(
                model="gpt-4",
                message_text=FOCUS_FILE_PATH_PROMPT.format(
                    format_example=FOCUS_FILE_PATH_EXAMPLE,
                ),
            ),
        )

        files = LLMOutputParser.parse_output(resp.message.content)["files"]
        for file in files:
            try:
                self.repo_manager.add_focus_file(file)
            except FileNotFoundError:
                logging.warning(f"File {file} does not exist.")

    def design_plan(self):
        """
        Design high level plan for feature development.
        """
        if len(self.repo_manager.focus_files) == 0:
            logging.warning("No focus files are set. Please set focus files first.")

        focus_files_dict = self.repo_manager.get_focus_files_content()
        focus_files_str = "\n".join(
            [
                f"{file_path}:\n{file_content}"
                for file_path, file_content in focus_files_dict.items()
            ]
        )

        # add focus files system message to openai node
        self.openai_node.add_single_message(
            Message(
                role="system",
                content=FOCUS_FILE_PROMPT.format(
                    focus_files=focus_files_str,
                ),
            )
        )

        resp = self.openai_node.chat(
            input=ChatInput(
                model="gpt-4",
                message_text=PLAN_PROMPT.format(
                    format_example=PLAN_FORMAT_EXAMPLE,
                ),
            ),
        )

        plan = LLMOutputParser.parse_output(resp.message.content)["plan"]
        for action, file_path, description in plan:
            self.plans.append(
                Plan(
                    action=action.lower(), file_path=file_path, description=description
                )
            )

    def implement(self):
        """
        Implement plan, generate a list of file actions.
        """
        for plan in self.plans:
            match plan.action:
                case "add":
                    file_action = self._add_file(plan)
                case "remove":
                    file_action = self._remove_file(plan)
                case "modify":
                    file_action = self._modify_file(plan)
                case _:
                    raise ValueError(f"Unknown action {plan.action}.")

            self.file_actions.append(file_action)
            yield file_action

    def apply_file_action(self):
        self.repo_manager.apply_file_actions(self.file_actions)
        self.file_actions = []

    def _add_file(self, plan: Plan):
        assert plan.action == "add"

        resp = self.openai_node.chat(
            input=ChatInput(
                model="gpt-4",
                message_text=ADD_FILE_PROMPT.format(
                    file_path=plan.file_path,
                    action_description=plan.description,
                ),
            )
        )

        content = resp.message.content

        while content.startswith('"') and content.endswith('"'):
            content = content[1:-1]

        return FileAction(
            action="add",
            file_path=plan.file_path,
            content=content,
        )

    def _remove_file(self, plan: Plan):
        return FileAction(
            action="remove",
            file_path=plan.file_path,
        )

    def _modify_file(self, plan: Plan):
        assert plan.action == "modify"

        resp = self.openai_node.chat(
            input=ChatInput(
                model="gpt-4",
                message_text=MODIFY_FILE_PROMPT.format(
                    file_path=plan.file_path,
                    file_content=self.repo_manager.get_file_content(
                        plan.file_path, limit_lines=5000
                    ),
                    action_description=plan.description,
                ),
            )
        )

        content = resp.message.content

        while content.startswith('"') and content.endswith('"'):
            content = content[1:-1]

        return FileAction(
            action="modify",
            file_path=plan.file_path,
            content=content,
        )

    def _init_openai_node(self):
        """
        Initialize OpenAI node.
        Add global system messages.
        """
        self.openai_node = OpenAINode()

        # software engineer prompt
        self.openai_node.add_single_message(
            Message(
                role="system",
                content=SDE_PROMPT,
            )
        )

    # Getter and Setter

    def add_focus_file(self, file_path):
        """
        Add file to repo.
        """
        self.repo_manager.add_focus_file(file_path)

    def remove_focus_file(self, file_path):
        """
        Remove file from repo.
        """
        self.repo_manager.remove_focus_file(file_path)

    def get_focus_files(self):
        """
        Get focus files.
        """
        return self.repo_manager.focus_files

    def clear_focus_files(self):
        """
        Clear focus files.
        """
        self.repo_manager.focus_files = {}

    def add_plan(self, plan: Plan):
        """
        Add plan to implement plan.
        """
        self.plans.append(plan)

    def remove_plan(self, plan: Plan):
        """
        Remove plan from implement plan.
        """
        self.plans.remove(plan)

    def get_plan(self):
        """
        Get feature implemetation plan.
        """
        return self.plans

    def set_plans(self, plans: list[Plan]):
        """
        Set feature implementation plan.
        """
        self.plans = plans

    def get_file_actions(self):
        """
        Get file actions.
        """
        return self.file_actions



import requests
from github import Github
from ..base_node import BaseNode, NodeConfig
from .github_model import (
    RepositoryInput,
    GetSpecificContentFileInput,
    CreateFileInput,
    EditFileInput,
    DeleteFileInput,
)

import os
from dotenv import load_dotenv

load_dotenv()  # This will load variables from .env into the environment

GITHUB_TOKEN = os.environ.get("GITHUB_TOKEN")

repository_node_config = {
    "name": "repository",
    "description": "A node for interacting with GitHub repositories and files.",
    "functions": {
        "get_repository_topics": "Get repository topics.",
        "get_count_of_stars": "Get count of stars.",
        "get_specific_content_file": "Get a specific content file.",
        "create_file": "Create a new file in the repository.",
        "edit_file": "Edit a file in the repository.",
        "delete_file": "Delete a file in the repository.",
        "get_list_of_open_issues": "Get a list of open issues.",
        "get_list_of_code_scanning_alerts": "Get a list of code scanning alerts.",
        "get_all_labels_of_repository": "Get all labels of the repository.",
        "get_contents_of_root_directory": "Get contents of the root directory.",
        "get_all_contents_recursively": "Get all contents recursively.",
        "get_top_10_referrers": "Get top 10 referrers.",
        "get_top_10_popular_contents": "Get top 10 popular contents.",
        "get_clone_and_view_data": "Get clone and view data.",
        "mark_notifications_as_read": "Mark notifications as read.",
    },
}

class GithubNode(BaseNode):
    def __init__(self, token: str):
        self.token = token
        self.g = Github(self.token)  # Initializing the GitHub instance with the token
        super().__init__()

class RepositoryNode(GithubNode):
    config: NodeConfig = NodeConfig(**repository_node_config)

    def __init__(self):
        token = os.environ.get("GITHUB_TOKEN")  # Retrieving the token from the environment
        if not token:
            raise ValueError("GITHUB_TOKEN is not set in the environment.")
        super().__init__(token)

    def get_repository_topics(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            topics = repo.get_topics()
            return topics
        except Exception as e:
            return str(e)

    def get_count_of_stars(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            stars_count = repo.stargazers_count
            return stars_count
        except Exception as e:
            return str(e)

    def get_specific_content_file(self, input: GetSpecificContentFileInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            content_file = repo.get_contents(input.path)
            return content_file.decoded_content.decode()
        except Exception as e:
            return str(e)

    def create_file(self, input: CreateFileInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            repo.create_file(input.path, input.message, input.content)
            return {"status": "File created successfully."}
        except Exception as e:
            return str(e)

    def edit_file(self, input: EditFileInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            content_file = repo.get_contents(input.path)
            repo.update_file(input.path, input.message, input.content, content_file.sha)
            return {"status": "File updated successfully."}
        except Exception as e:
            return str(e)

    def delete_file(self, input: DeleteFileInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            content_file = repo.get_contents(input.path)
            repo.delete_file(input.path, input.message, content_file.sha)
            return {"status": "File deleted successfully."}
        except Exception as e:
            return str(e)

    def get_list_of_open_issues(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            issues = repo.get_issues(state='open')
            return [issue.title for issue in issues]
        except Exception as e:
            return str(e)

    # def get_list_of_code_scanning_alerts(self, input: RepositoryInput):
    #     # Note: This function might require direct API calls as it's not clearly supported in PyGithub at the moment.
    #     raise NotImplementedError("This function is not implemented yet.")

    def get_list_of_code_scanning_alerts(self, input: RepositoryInput):
        try:
            headers = {
                "Authorization": f"token {self.g.get_user().get_access_token().token}", 
                "Accept": "application/vnd.github.v3+json"
            }
            url = f"https://api.github.com/repos/{input.owner}/{input.repo_name}/code-scanning/alerts"
            response = requests.get(url, headers=headers)
            response.raise_for_status()  # Will raise an exception for 4xx and 5xx responses

            return response.json()

        except requests.RequestException as e:
            return {"error": str(e)}


    def get_all_labels_of_repository(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            labels = repo.get_labels()
            return [label.name for label in labels]
        except Exception as e:
            return str(e)

    def get_contents_of_root_directory(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            contents = repo.get_contents("")
            return [content.name for content in contents]
        except Exception as e:
            return str(e)

    def get_all_contents_recursively(self, input: RepositoryInput):
        # This will likely be a recursive method since GitHub API doesn't directly support fetching all contents recursively.
        raise NotImplementedError("This function is not implemented yet.")

    def get_top_10_referrers(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            referrers = repo.get_top_referrers()[:10]
            return [{"referrer": referrer.referrer, "count": referrer.count, "uniques": referrer.uniques} for referrer in referrers]
        except Exception as e:
            return str(e)

    def get_top_10_popular_contents(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            popular_contents = repo.get_top_paths()[:10]
            return [{"path": content.path, "count": content.count, "uniques": content.uniques} for content in popular_contents]
        except Exception as e:
            return str(e)

    def get_clone_and_view_data(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            clones = repo.get_clones_traffic()
            views = repo.get_views_traffic()
            return {"clones": clones, "views": views}
        except Exception as e:
            return str(e)

    def mark_notifications_as_read(self, input: RepositoryInput):
        try:
            notifications = self.g.get_user().get_notifications()
            for notification in notifications:
                if notification.repository.full_name == f"{input.owner}/{input.repo_name}":
                    notification.mark_as_read()
            return {"status": "Notifications marked as read."}
        except Exception as e:
            return str(e)


RESPONSE_PROMPT="""
As a polite AI assistant, you need to observe the required parameter information and then ask the user questions to get the corresponding parameter information.
Please provide specific information or feedback based on the following parameter information parameter_info. Your answer should be clear and unambiguous, relevant to the information provided by the parameters, and provide appropriate details and examples.
"""

RESPONSE_EXAMPLE_PROMPT="""
Here is some examples

Parameter information (parameter_info):
[{'name': 'name', 'required': True, 'parameter_schema': {'type': 'string', 'description': 'The name of this people.', 'properties': None, 'items': None}},{'name': 'age', 'required': True, 'parameter_schema': {'type': 'int', 'description': 'The age of this people.', 'properties': None, 'items': None}},{'name': 'email', 'required': True, 'parameter_schema': {'type': 'string', 'description': 'The email of this people.', 'properties': None, 'items': None}}]

Response sentence:
To proceed, could you please provide the name, age, and email of the person in question? These details are essential for us to move forward.

Parameter information (parameter_info):
[{'name': 'w', 'required': True, 'parameter_schema': {'type': 'string', 'description': 'What kind of weather?', 'properties': None, 'items': None}},{'name': 'age', 'required': True, 'parameter_schema': {'type': 'int', 'description': 'The age of this people.', 'properties': None, 'items': None}},{'name': 'email', 'required': True, 'parameter_schema': {'type': 'string', 'description': 'The email of this people.', 'properties': None, 'items': None}}]

Response sentence:
Could you please tell me what type of weather you are experiencing, how old the person in question is, and their email address?

Parameter information (parameter_info):
[{'name': 'x', 'required': True, 'parameter_schema': {'type': 'string', 'description': 'input x content', 'properties': None, 'items': None}},{'name': 'age', 'required': True, 'parameter_schema': {'type': 'int', 'description': 'The age of this people.', 'properties': None, 'items': None}},{'name': 'email', 'required': True, 'parameter_schema': {'type': 'string', 'description': 'The email of this people.', 'properties': None, 'items': None}}]

Response sentence:
Could you please provide the content for 'x' (a string input), the age of the person involved (an integer), and their email address (a string input)?
"""

RESPONSE_PROMPT_HINT = """
please output a sentence to user.
The goal is for the user to answer based on the parameters
"""


from ..base_node import BaseNode, NodeConfig
from .github_node import GithubNode
from .github_model import (
    RepositoryInput,
    GetBranchInput,
    GetBranchHeadCommitInput,
    GetBranchProtectionStatusInput,
    GetBranchRequiredStatusChecksInput,
)

github_branch_node_config = {
    "name": "github_branch",
    "description": "A node for interacting with GitHub branches.",
    "functions": {
        "get_list_of_branches": "Get a list of branches.",
        "get_branch": "Get a specific branch.",
        "get_head_commit_of_branch": "Get the HEAD commit of a branch.",
        "get_protection_status_of_branch": "Get the protection status of a branch.",
        "see_required_status_checks_of_branch": "See the required status checks of a branch.",
    },
}


class GithubBranchNode(GithubNode):
    config: NodeConfig = NodeConfig(**github_branch_node_config)

    def __init__(self):
        super().__init__()

    def get_list_of_branches(self, input: RepositoryInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            branches = repo.get_branches()
            return [branch.name for branch in branches]
        except Exception as e:
            return str(e)

    def get_branch(self, input: GetBranchInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            branch = repo.get_branch(branch=input.branch_name)
            return branch.name
        except Exception as e:
            return str(e)

    def get_head_commit_of_branch(self, input: GetBranchHeadCommitInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            branch = repo.get_branch(branch=input.branch_name)
            return branch.commit.sha
        except Exception as e:
            return str(e)

    def get_protection_status_of_branch(self, input: GetBranchProtectionStatusInput):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            protection = repo.get_branch_protection(branch=input.branch_name)
            return protection.url
        except Exception as e:
            return str(e)

    def see_required_status_checks_of_branch(
        self, input: GetBranchRequiredStatusChecksInput
    ):
        try:
            repo = self.g.get_repo(f"{input.owner}/{input.repo_name}")
            protection = repo.get_branch_protection(branch=input.branch_name)
            return protection.required_status_checks
        except Exception as e:
            return str(e)



import sys

from io import StringIO
from typing import Optional


class PythonREPL:
    """
    Simple python REPL.
    """

    _globals: dict[str, any]
    _locals: dict[str, any]

    def __init__(self, _globals: Optional[dict] = None, _locals: Optional[dict] = None):
        self._globals = _globals if _globals is not None else {}
        self._locals = _locals if _locals is not None else {}

    def run(self, command: str) -> str:
        old_stdout = sys.stdout
        sys.stdout = mystdout = StringIO()
        try:
            exec(command, self._globals, self._locals)
            sys.stdout = old_stdout
            output = mystdout.getvalue()
        except Exception as e:
            sys.stdout = old_stdout
            output = str(e)
        return output



from __future__ import annotations  # for type hinting the class itself

from typing import Optional, Literal
from enum import Enum
from pydantic import BaseModel, Field

# model for tool and tool entity


class ObjectType(str, Enum):
    STRING = "string"
    NUMBER = "number"
    BOOLEAN = "boolean"
    OBJECT = "object"
    ARRAY = "array"


class Schema(BaseModel):
    type: Literal['string', 'number', 'boolean', 'object', 'array'] = Field(description="参数类型")
    description: Optional[str] = Field(description="参数描述")
    properties: Optional[dict[str, Schema]] = Field(description="参数类型为 object 时的元素类型")
    items: Optional[list[Schema]] = Field(description="参数类型为 array 时的元素类型")


class Parameter(BaseModel):
    name: str = Field(description="参数名")
    required: bool = Field(description="是否必须")
    parameter_schema: Schema = Field(description="参数schema")


class Response(BaseModel):
    description: str = Field(description="响应描述")
    content: dict[str, Schema] = Field(description="响应内容")


# model for stateful tool entity

class Stage(BaseModel):
    name: str = Field(description="Stage 名称")
    next_stage_entry: dict[str, list[Parameter]] = Field(
        description="下一个可能的 Stage 名以及对应需要传入的参数列表"
    )
    need_llm_generate_parameters: bool = Field(
        description="在这个 Stage 输入时是否需要 LLM 生成参数"
    )
    need_llm_generate_response: bool = Field(
        description="在这个 Stage 输入时是否需要 LLM 生成回复内容"
    )



import ast

from typing import Optional

from ...nodes.base_node import BaseNode, NodeConfig
from ...nodes.code_runner.code_runner_model import (
    RunCodeInput,
    RunCodeFromFileInput,
)
from ....utils.code_executor.python_process import PythonProcess
from ....utils.code_executor.python_repl import PythonREPL

code_runner_config = {
    "name": "code_runner",
    "description": "A simple node that run python code.",
    "functions": {
        "run_code": "Run python code in string format.",
        "run_code_from_file": "Run python code in specific files.",
    },
}


class CodeRunnerNode(BaseNode):
    config: NodeConfig = NodeConfig(**code_runner_config)

    pythonREPL: PythonREPL

    def __init__(self):
        super().__init__()
        self.pythonREPL = None

    # TODO: check if the input is valid

    def run_code(self, input: RunCodeInput):
        if self.pythonREPL is None:
            self.init_python_repl()
        return self.pythonREPL.run(input.code)

    def run_code_from_file(self, input: RunCodeFromFileInput):
        working_dir = input.working_dir
        if ' ' in input.working_dir:
            working_dir = f'"{input.working_dir}"'

        file_path = input.file_path
        if ' ' in input.file_path:
            file_path = f'"{input.file_path}"'

        proc = PythonProcess(
            working_dir=working_dir,
            python_file_path=file_path,
        )
        return proc.run()

    def init_python_repl(
        self, globals_: Optional[dict] = None, locals_: Optional[dict] = None
    ):
        self.pythonREPL = PythonREPL(globals_, locals_)



import yaml
import os
import inspect
from pydantic import BaseModel, Field

from .tool_entity import (
    BaseToolEntity,
    FunctionToolEntity,
    State
)
from .model import Parameter, Response

from .swe_tool_entity import SWEToolEntity
from .example_stateful_tool_entity import ExampleStatefulToolEntity


# 示例工具函数
def serve_code_interpreter(code: str) -> dict[str, any]:
    from ...nodes.code_runner.code_runner import CodeRunnerNode, RunCodeInput

    code_runner_node = CodeRunnerNode()
    code_runner_node.init_python_repl()
    res = code_runner_node.run_code(RunCodeInput(code=code))

    return {
        "type": "success",
        "content": {
            "result": res,
        },
    }


FUNCTION_TOOL_ENTITIES = {
    "code_interpreter": serve_code_interpreter,
}

STATEFUL_TOOL_ENTITIES = {
    "example_stateful_tool": ExampleStatefulToolEntity,
    "swe_tool":SWEToolEntity,
}

def register_function_tool(func):
    FUNCTION_TOOL_ENTITIES[func.__name__] = func
    return func

def register_stateful_tool(cls):
    STATEFUL_TOOL_ENTITIES[cls.__name__] = cls
    return cls

class ToolConfig(BaseModel):
    name: str = Field(description="工具名称")
    entity_name: str = Field(description="工具实体名称")
    summary: str = Field(description="工具描述")
    parameters: list[Parameter] = Field(description="参数列表")
    responses: dict[str, Response] = Field(description="响应列表")


class Tool:
    config: ToolConfig
    entity: BaseToolEntity

    def __init__(self, config: ToolConfig):
        self.config = config
        entity_name = config.entity_name

        if entity_name in FUNCTION_TOOL_ENTITIES:
            self.entity = FunctionToolEntity(FUNCTION_TOOL_ENTITIES[entity_name])
        elif entity_name in STATEFUL_TOOL_ENTITIES:
            self.entity = STATEFUL_TOOL_ENTITIES[entity_name]()
        else:
            raise Exception(f"Tool entity {entity_name} not found.")

    # TODO: response check and type convert
    def call(self, **kwargs):
        return self.entity.call(**kwargs)

    def need_llm_generate_parameters(self) -> bool:
        return self.entity.need_llm_generate_parameters()

    def need_llm_generate_response(self) -> bool:
        return self.entity.need_llm_generate_response()

    def has_done(self) -> bool:
        return self.entity.current_state() == State.DONE

class Tools:
    tools: dict[str, Tool]

    def __init__(self,yaml_file_path:str):
        self.tools = {}
        # 获取调用此方法的栈帧
        stack = inspect.stack()
        caller_frame = stack[2]
        # 获取调用者的文件路径
        caller_path = caller_frame.filename
        # 获取调用者的目录路径
        caller_dir = os.path.dirname(caller_path)
        # 构建 openai.yaml 文件的绝对路径
        yaml_file_path = os.path.join(caller_dir, yaml_file_path)
        tools_yaml_path = yaml_file_path
        # 读取 tools.yaml 文件，初始化所有 tools
        with open(tools_yaml_path, "r") as f:
            config_obj = yaml.safe_load(f)
            for tool_name, tool_config in config_obj["tools"].items():
                self.tools[tool_name] = Tool(config=ToolConfig(**tool_config))

    def get_tool(self, tool_name: str) -> Tool:
        # 找到对应的工具
        tool = self.tools.get(tool_name)
        if tool is None:
            raise ValueError(f"No tool named {tool_name} found.")

        return tool

    def get_tool_summary(self, tool_name: str) -> str:
        # 在 tools.yaml 文件中找到对应的工具
        tool = self.tools.get(tool_name)
        if tool is None:
            raise ValueError(f"No tool named {tool_name} found.")

        return tool.config.summary

    def get_tools_list_summary(self, tools_list: list[str]) -> dict[str, str]:
        tools_summary = {}
        for tool_name in tools_list:
            summary = self.get_tool_summary(tool_name)
            tools_summary[tool_name] = summary
        return tools_summary


from abc import ABC, abstractmethod
from pydantic import BaseModel, Field


class AgentConfig(BaseModel):
    name: str = Field(description="Agent 名称")


class BaseAgent(ABC):
    AgentConfig: AgentConfig

